@@ -134,6 +134,12 @@ struct _WnckTaskClass
   GObjectClass parent_class;
 };
 
+typedef struct _skipped_window
+{
+  WnckWindow *window;
+  gulong tag;
+} skipped_window;
+
 struct _WnckTasklistPrivate
 {
   WnckScreen *screen;
@@ -150,6 +156,11 @@ struct _WnckTasklistPrivate
   /* Not handled by update_lists */
   GList *startup_sequences;
   
+  /* windows with _NET_WM_STATE_SKIP_TASKBAR set; connected to 
+   * "state_changed" signal, but excluded from tasklist.
+   */
+  GList *skipped_windows; 
+
   GHashTable *class_group_hash;
   GHashTable *win_hash;
   
@@ -185,6 +196,7 @@ struct _WnckTasklistPrivate
 
   gint monitor_num;
   GdkRectangle monitor_geometry;
+  GtkReliefStyle relief;
 };
 
 
@@ -203,13 +215,15 @@ static WnckTask *wnck_task_new_from_startup_sequence (WnckTasklist      *tasklis
 static gboolean wnck_task_get_needs_attention (WnckTask *task);
 
 
-static char      *wnck_task_get_text (WnckTask *task);
+static char      *wnck_task_get_text (WnckTask *task, gboolean icon_text);
 static GdkPixbuf *wnck_task_get_icon (WnckTask *task);
 static gint       wnck_task_compare  (gconstpointer  a,
 				      gconstpointer  b);
 static void       wnck_task_update_visible_state (WnckTask *task);
-
-
+static void       wnck_task_state_changed        (WnckWindow      *window,
+                                                  WnckWindowState  changed_mask, 
+                                                  WnckWindowState  new_state,
+                                                  gpointer         data);
 static void wnck_tasklist_init        (WnckTasklist      *tasklist);
 static void wnck_tasklist_class_init  (WnckTasklistClass *klass);
 static void wnck_tasklist_finalize    (GObject        *object);
@@ -280,53 +294,10 @@ static gpointer tasklist_parent_class;
  */
 static GSList *tasklist_instances;
 
-/**
- * make_gtk_label_bold.
- *
- * Switches the font of label to a bold equivalent.
- * @label: The label.
- **/
-static void
-make_gtk_label_bold (GtkLabel *label)
-{
-  PangoFontDescription *font_desc;
+static GType
+wnck_task_get_type (void) G_GNUC_CONST;
 
-  font_desc = pango_font_description_new ();
-
-  pango_font_description_set_weight (font_desc,
-                                     PANGO_WEIGHT_BOLD);
-
-  /* This will only affect the weight of the font, the rest is
-   * from the current state of the widget, which comes from the
-   * theme or user prefs, since the font desc only has the
-   * weight flag turned on.
-   */
-  gtk_widget_modify_font (GTK_WIDGET (label), font_desc);
-
-  pango_font_description_free (font_desc);
-}
-
-static void
-make_gtk_label_normal (GtkLabel *label)
-{
-  PangoFontDescription *font_desc;
-
-  font_desc = pango_font_description_new ();
-
-  pango_font_description_set_weight (font_desc,
-                                     PANGO_WEIGHT_NORMAL);
-
-  /* This will only affect the weight of the font, the rest is
-   * from the current state of the widget, which comes from the
-   * theme or user prefs, since the font desc only has the
-   * weight flag turned on.
-   */
-  gtk_widget_modify_font (GTK_WIDGET (label), font_desc);
-
-  pango_font_description_free (font_desc);
-}
-
-GType
+static GType
 wnck_task_get_type (void)
 {
   static GType object_type = 0;
@@ -346,6 +317,7 @@ wnck_task_get_type (void)
         sizeof (WnckTask),
         0,              /* n_preallocs */
         (GInstanceInitFunc) wnck_task_init,
+        NULL            /* value_table */
       };
       
       object_type = g_type_register_static (G_TYPE_OBJECT,
@@ -642,6 +614,7 @@ wnck_tasklist_get_type (void)
         sizeof (WnckTasklist),
         0,              /* n_preallocs */
         (GInstanceInitFunc) wnck_tasklist_init,
+        NULL            /* value_table */
       };
       
       object_type = g_type_register_static (GTK_TYPE_CONTAINER,
@@ -678,15 +651,13 @@ wnck_tasklist_init (WnckTasklist *tasklist)
   tasklist->priv->idle_callback_tag = 0;
 
   tasklist->priv->monitor_num = -1;
+  tasklist->priv->relief = GTK_RELIEF_NORMAL;
+
+  tasklist->priv->skipped_windows = NULL;
 
   atk_obj = gtk_widget_get_accessible (widget);
   atk_object_set_name (atk_obj, _("Window List"));
   atk_object_set_description (atk_obj, _("Tool to switch between visible windows"));
-
-  tasklist_instances = g_slist_append (tasklist_instances, tasklist);
-  g_slist_foreach (tasklist_instances,
-		   (GFunc) wnck_tasklist_update_lists,
-		   NULL);
 }
 
 static void
@@ -709,21 +680,41 @@ wnck_tasklist_class_init (WnckTasklistClass *klass)
   container_class->remove = wnck_tasklist_remove;
 }
 
+static void
+wnck_tasklist_free_skipped_windows (WnckTasklist  *tasklist)
+{
+  GList *l;
+  
+  l = tasklist->priv->skipped_windows;
+  
+  while (l != NULL)
+    {
+      skipped_window *skipped = (skipped_window*) l->data;
+      g_signal_handler_disconnect (skipped->window, skipped->tag);
+      g_object_unref (skipped->window);
+      g_free (skipped);
+      l = l->next;
+    }
+  
+  g_list_free (tasklist->priv->skipped_windows);
+}
+
 static void
 wnck_tasklist_finalize (GObject *object)
 {
   WnckTasklist *tasklist;
 
   tasklist = WNCK_TASKLIST (object);
-  
-  tasklist_instances = g_slist_remove (tasklist_instances, tasklist);
-  g_slist_foreach (tasklist_instances,
-		   (GFunc) wnck_tasklist_update_lists,
-		   NULL);
 
   if (tasklist->priv->free_icon_loader_data != NULL)
     (* tasklist->priv->free_icon_loader_data) (tasklist->priv->icon_loader_data);
   
+  if (tasklist->priv->skipped_windows)
+    {
+      wnck_tasklist_free_skipped_windows (tasklist);
+      tasklist->priv->skipped_windows = NULL;
+    }
+  
   wnck_tasklist_disconnect_screen (tasklist);
 
   /* Tasks should have gone away due to removing their
@@ -775,6 +766,27 @@ wnck_tasklist_set_grouping (WnckTasklist            *tasklist,
   gtk_widget_queue_resize (GTK_WIDGET (tasklist));
 }
 
+static void
+wnck_tasklist_set_relief_callback (WnckWindow   *win,
+				   WnckTask     *task,
+				   WnckTasklist *tasklist)
+{
+  gtk_button_set_relief (GTK_BUTTON (task->button), tasklist->priv->relief);
+}
+
+void
+wnck_tasklist_set_button_relief (WnckTasklist *tasklist, GtkReliefStyle relief)
+{
+  if (relief == tasklist->priv->relief)
+    return;
+
+  tasklist->priv->relief = relief;
+
+  g_hash_table_foreach (tasklist->priv->win_hash,
+                        (GHFunc) wnck_tasklist_set_relief_callback,
+                        tasklist);
+}
+
 void
 wnck_tasklist_set_switch_workspace_on_unminimize (WnckTasklist  *tasklist,
 						  gboolean       switch_workspace_on_unminimize)
@@ -1375,7 +1387,10 @@ wnck_tasklist_realize (GtkWidget *widget)
   
   (* GTK_WIDGET_CLASS (tasklist_parent_class)->realize) (widget);
 
-  wnck_tasklist_update_lists (tasklist);
+  tasklist_instances = g_slist_append (tasklist_instances, tasklist);
+  g_slist_foreach (tasklist_instances,
+		   (GFunc) wnck_tasklist_update_lists,
+		   NULL);
 }
 
 static void
@@ -1391,6 +1406,11 @@ wnck_tasklist_unrealize (GtkWidget *widget)
 #endif
   
   (* GTK_WIDGET_CLASS (tasklist_parent_class)->unrealize) (widget);
+  
+  tasklist_instances = g_slist_remove (tasklist_instances, tasklist);
+  g_slist_foreach (tasklist_instances,
+		   (GFunc) wnck_tasklist_update_lists,
+		   NULL);
 }
 
 static void
@@ -1639,6 +1659,13 @@ wnck_tasklist_free_tasks (WnckTasklist *tasklist)
 	  gtk_widget_destroy (task->button);
 	}
     }
+  
+  if (tasklist->priv->skipped_windows)
+    {
+      wnck_tasklist_free_skipped_windows (tasklist);
+      tasklist->priv->skipped_windows = NULL;
+    }
+  
   g_assert (tasklist->priv->class_groups == NULL);
   g_assert (g_hash_table_size (tasklist->priv->class_group_hash) == 0);
 }
@@ -1648,14 +1675,12 @@ wnck_tasklist_free_tasks (WnckTasklist *tasklist)
  * This function determines if a window should be included in the tasklist.
  */
 static gboolean
-wnck_tasklist_include_window (WnckTasklist *tasklist, WnckWindow *win)
+tasklist_include_window_ignoring_skip_taskbar (WnckTasklist *tasklist,
+                                               WnckWindow *win)
 {
   WnckWorkspace *active_workspace;
   int x, y, w, h;
 
-  if (wnck_window_get_state (win) & WNCK_WINDOW_STATE_SKIP_TASKLIST)
-    return FALSE;
-
   if (tasklist->priv->monitor_num != -1)
     {
       wnck_window_get_geometry (win, &x, &y, &w, &h);
@@ -1684,6 +1709,15 @@ wnck_tasklist_include_window (WnckTasklist *tasklist, WnckWindow *win)
   return wnck_window_is_in_viewport (win, active_workspace);
 }
 
+static gboolean
+wnck_tasklist_include_window (WnckTasklist *tasklist, WnckWindow *win)
+{
+  if (wnck_window_get_state (win) & WNCK_WINDOW_STATE_SKIP_TASKLIST)
+    return FALSE;
+
+  return tasklist_include_window_ignoring_skip_taskbar (tasklist, win);
+}
+
 static void
 wnck_tasklist_update_lists (WnckTasklist *tasklist)
 {
@@ -1754,6 +1788,18 @@ wnck_tasklist_update_lists (WnckTasklist *tasklist)
 	  
 	  class_group_task->windows = g_list_prepend (class_group_task->windows, win_task);
 	}
+      else if (tasklist_include_window_ignoring_skip_taskbar (tasklist, win))
+        {
+          skipped_window *skipped = g_new0 (skipped_window, 1);
+          skipped->window = g_object_ref (win);
+          skipped->tag = g_signal_connect (G_OBJECT (win),
+                                           "state_changed",
+                                           G_CALLBACK (wnck_task_state_changed),
+                                           tasklist);
+          tasklist->priv->skipped_windows = 
+            g_list_prepend (tasklist->priv->skipped_windows,
+                            (gpointer) skipped);
+        }
       
       l = l->next;
     }
@@ -1879,12 +1925,14 @@ wnck_tasklist_active_window_changed (WnckScreen   *screen,
 				     WnckTasklist *tasklist)
 {
   WnckWindow *active_window;
-  WnckTask *active_task;
-  
-  active_window = wnck_screen_get_active_window (screen);
+  WnckTask *active_task = NULL;
 
-  active_task = g_hash_table_lookup (tasklist->priv->win_hash,
-				     active_window);
+  /* FIXME: check for group modal window */
+  for (active_window = wnck_screen_get_active_window (screen);
+       active_window && !active_task;
+       active_window = wnck_window_get_transient(active_window))
+    active_task = g_hash_table_lookup (tasklist->priv->win_hash,
+                                       active_window);
 
   wnck_tasklist_change_active_task (tasklist, active_task);
 }
@@ -2122,14 +2170,15 @@ wnck_tasklist_activate_task_window (WnckTask *task,
       window_ws = wnck_window_get_workspace (task->window);
       if (window_ws &&
           active_ws != window_ws &&
-          tasklist->priv->switch_workspace_on_unminimize)
-        wnck_window_move_to_workspace (task->window, active_ws);
+          !tasklist->priv->switch_workspace_on_unminimize)
+        wnck_workspace_activate (window_ws, timestamp);
 
       wnck_window_activate_transient (task->window, timestamp);
     }
   else
     {
-      if (task->was_active || wnck_window_transient_is_active (task->window))
+      if (task->was_active || 
+          wnck_window_transient_is_most_recently_activated (task->window))
 	{
 	  task->was_active = FALSE;
 	  wnck_window_minimize (task->window);
@@ -2137,6 +2186,16 @@ wnck_tasklist_activate_task_window (WnckTask *task,
 	}
       else
 	{
+          WnckWorkspace *window_ws;
+          
+          /* FIXME: THIS IS SICK AND WRONG AND BUGGY.  See the end of
+           * http://mail.gnome.org/archives/wm-spec-list/2005-July/msg00032.html
+           * There should only be *one* activate call.
+           */
+          window_ws = wnck_window_get_workspace (task->window);
+          if (window_ws)
+            wnck_workspace_activate (window_ws, timestamp);
+
           wnck_window_activate_transient (task->window, timestamp);
         }
     }
@@ -2238,10 +2297,10 @@ wnck_task_popup_menu (WnckTask *task,
     {
       win_task = WNCK_TASK (l->data);
       
-      text = wnck_task_get_text (win_task);
+      text = wnck_task_get_text (win_task, FALSE);
       menu_item = gtk_image_menu_item_new_with_label (text);
       if (wnck_task_get_needs_attention (win_task)) 
-        make_gtk_label_bold (GTK_LABEL (GTK_BIN (menu_item)->child));
+        _make_gtk_label_bold (GTK_LABEL (GTK_BIN (menu_item)->child));
       g_free (text);
       
       pixbuf = wnck_task_get_icon (win_task);
@@ -2358,7 +2417,7 @@ wnck_task_button_toggled (GtkButton *button,
 }
 
 static char *
-wnck_task_get_text (WnckTask *task)
+wnck_task_get_text (WnckTask *task, gboolean icon_text)
 {
   WnckWindowState state;
   const char *name;
@@ -2376,8 +2435,12 @@ wnck_task_get_text (WnckTask *task)
 				g_list_length (task->windows));
 
     case WNCK_TASK_WINDOW:
+      if (icon_text)
+        name = wnck_window_get_icon_name (task->window);
+      else 
+        name = wnck_window_get_name (task->window);
+       
       state = wnck_window_get_state (task->window);
-      name = wnck_window_get_name (task->window);
       
       if (state & WNCK_WINDOW_STATE_SHADED)
 	return g_strdup_printf ("=%s=", name);
@@ -2594,22 +2657,27 @@ wnck_task_update_visible_state (WnckTask *task)
   if (pixbuf)
     g_object_unref (pixbuf);
 
-  text = wnck_task_get_text (task);
+  text = wnck_task_get_text (task, TRUE);
   if (text != NULL)
     {
       gtk_label_set_text (GTK_LABEL (task->label), text);
       if (wnck_task_get_needs_attention (task))
         {
-          make_gtk_label_bold ((GTK_LABEL (task->label)));
+          _make_gtk_label_bold ((GTK_LABEL (task->label)));
           wnck_task_queue_glow (task);
         }
       else
         {
-          make_gtk_label_normal ((GTK_LABEL (task->label)));
+          _make_gtk_label_normal ((GTK_LABEL (task->label)));
           wnck_task_stop_glow (task);
         }
-      gtk_tooltips_set_tip (task->tasklist->priv->tooltips, task->button, text, NULL);
       g_free (text);
+      text = wnck_task_get_text (task, FALSE);
+      if (text != NULL)
+        {
+          gtk_tooltips_set_tip (task->tasklist->priv->tooltips, task->button, text, NULL);
+          g_free (text);
+        }
     }
 
   gtk_widget_queue_resize (GTK_WIDGET (task->tasklist));
@@ -2634,10 +2702,11 @@ wnck_task_state_changed (WnckWindow     *window,
       (changed_mask & WNCK_WINDOW_STATE_DEMANDS_ATTENTION) ||
       (changed_mask & WNCK_WINDOW_STATE_URGENT))
     {
-      WnckTask *win_task;
+      WnckTask *win_task = NULL;
 
-      win_task = g_hash_table_lookup (tasklist->priv->win_hash,
-						window);
+      /* FIXME: Handle group modal dialogs */
+      for (; window && !win_task; window = wnck_window_get_transient (window))
+        win_task = g_hash_table_lookup (tasklist->priv->win_hash, window);
 
       if (win_task)
 	{
@@ -2645,7 +2714,9 @@ wnck_task_state_changed (WnckWindow     *window,
 
 	  wnck_task_update_visible_state (win_task);
 	  
-	  class_group_task = g_hash_table_lookup (tasklist->priv->class_group_hash, win_task->class_group);
+	  class_group_task =
+            g_hash_table_lookup (tasklist->priv->class_group_hash,
+                                 win_task->class_group);
 
 	  if (class_group_task)
 	    wnck_task_update_visible_state (class_group_task);
@@ -2810,7 +2881,7 @@ wnck_task_expose (GtkWidget        *widget,
                   gpointer          data);
 
 static void
-wnck_task_create_widgets (WnckTask *task)
+wnck_task_create_widgets (WnckTask *task, GtkReliefStyle relief)
 {
   GtkWidget *hbox;
   GdkPixbuf *pixbuf;
@@ -2824,6 +2895,9 @@ wnck_task_create_widgets (WnckTask *task)
     task->button = gtk_button_new ();
   else
     task->button = gtk_toggle_button_new ();
+
+  gtk_button_set_relief (GTK_BUTTON (task->button), relief);
+
   task->button_activate = 0;
   g_object_set_qdata (G_OBJECT (task->button),
 		      disable_sound_quark, GINT_TO_POINTER (TRUE));
@@ -2848,7 +2922,7 @@ wnck_task_create_widgets (WnckTask *task)
   
   gtk_widget_show (task->image);
 
-  text = wnck_task_get_text (task);
+  text = wnck_task_get_text (task, TRUE);
   task->label = gtk_label_new (text);
   gtk_misc_set_alignment (GTK_MISC (task->label), 0.0, 0.5);
   gtk_label_set_ellipsize (GTK_LABEL (task->label),
@@ -2857,7 +2931,7 @@ wnck_task_create_widgets (WnckTask *task)
 
   if (wnck_task_get_needs_attention (task))
     {
-      make_gtk_label_bold ((GTK_LABEL (task->label)));
+      _make_gtk_label_bold ((GTK_LABEL (task->label)));
       wnck_task_queue_glow (task);
     }
 
@@ -2868,7 +2942,9 @@ wnck_task_create_widgets (WnckTask *task)
 
   gtk_container_add (GTK_CONTAINER (task->button), hbox);
   gtk_widget_show (hbox);
+  g_free (text);
   
+  text = wnck_task_get_text (task, FALSE);
   gtk_tooltips_set_tip (task->tasklist->priv->tooltips, task->button, text, NULL);
   g_free (text);
   
@@ -3112,7 +3188,7 @@ wnck_task_new_from_window (WnckTasklist *tasklist,
   task->class_group = g_object_ref (wnck_window_get_class_group (window));
   task->tasklist = tasklist;
   
-  wnck_task_create_widgets (task);
+  wnck_task_create_widgets (task, tasklist->priv->relief);
 
   remove_startup_sequences_for_window (tasklist, window);
   
@@ -3132,7 +3208,7 @@ wnck_task_new_from_class_group (WnckTasklist   *tasklist,
   task->class_group = g_object_ref (class_group);
   task->tasklist = tasklist;
 
-  wnck_task_create_widgets (task);
+  wnck_task_create_widgets (task, tasklist->priv->relief);
 
   return task;
 }
@@ -3153,7 +3229,7 @@ wnck_task_new_from_startup_sequence (WnckTasklist      *tasklist,
   sn_startup_sequence_ref (task->startup_sequence);
   task->tasklist = tasklist;
   
-  wnck_task_create_widgets (task);
+  wnck_task_create_widgets (task, tasklist->priv->relief);
 
   return task;
 }
