@@ -111,11 +111,14 @@ item_activated_callback (GtkWidget *menu_item,
   switch (action)
     {
     case CLOSE:
-      wnck_window_close (amd->window);
+      /* In an activate callback, so gtk_get_current_event_time() suffices */
+      wnck_window_close (amd->window,
+			 gtk_get_current_event_time ());
       break;
     case MINIMIZE:
       if (wnck_window_is_minimized (amd->window))
-        wnck_window_unminimize (amd->window);
+        wnck_window_unminimize (amd->window,
+                                gtk_get_current_event_time ());
       else
         wnck_window_minimize (amd->window);
       break;
@@ -250,14 +253,14 @@ update_menu_state (ActionMenuData *amd)
 
   if (wnck_window_is_pinned (amd->window))
     {
-      set_item_text (amd->pin_item, _("Only on _This Workspace"));
+      set_item_text (amd->pin_item, _("_Only on This Workspace"));
       set_item_stock (amd->pin_item, NULL);
       gtk_widget_set_sensitive (amd->pin_item,
                                 (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
     }
   else
     {
-      set_item_text (amd->pin_item, _("Put on _All Workspaces"));
+      set_item_text (amd->pin_item, _("_Always on Visible Workspace"));
       set_item_stock (amd->pin_item, NULL);
       gtk_widget_set_sensitive (amd->pin_item,
                                 (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
@@ -441,7 +444,8 @@ wnck_create_window_action_menu (WnckWindow *window)
   ActionMenuData *amd;
   GtkWidget *separator;
   int num_workspaces, present_workspace, i;
- 
+  WnckWorkspace *workspace;
+
   _wnck_stock_icons_init ();
   
   amd = g_new0 (ActionMenuData, 1);
@@ -522,8 +526,13 @@ wnck_create_window_action_menu (WnckWindow *window)
                          amd->workspace_item);
 
   num_workspaces = wnck_screen_get_workspace_count (wnck_window_get_screen (amd->window));
-  present_workspace = wnck_workspace_get_number (wnck_window_get_workspace (amd->window));
- 
+  workspace = wnck_window_get_workspace (amd->window);
+
+  if (workspace)
+    present_workspace = wnck_workspace_get_number (workspace);
+  else 
+    present_workspace = -1;
+
   for (i = 0; i < num_workspaces; i++)
     {
       char *name, *label;
